/*
    Sequencer.cc
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <cstdio>

#include <iostream>

#include "Sequencer.h"
#include "MIDI.h"

using namespace std;

const int Sequencer::block2Part[32] =
{
  9, 0, 1, 2, 3, 4, 5, 6, 7, 8,10,11,12,13,14,15,
  25,16,17,18,19,20,21,22,23,24,26,27,28,29,30,31
};

const int Sequencer::block2Port[32] =
{
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,//MIDI::PORT_A
  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,//MIDI::PORT_B
};

const int Sequencer::block2Channel[32] =
{
  9, 0, 1, 2, 3, 4, 5, 6, 7, 8,10,11,12,13,14,15,
  9, 0, 1, 2, 3, 4, 5, 6, 7, 8,10,11,12,13,14,15
};


Sequencer::Sequencer (State* globalState, Option* midiOption):
  state (globalState), cacheState (*globalState), option (midiOption),
  __isSeeking (false)
{
  reset ();
}

Sequencer::~Sequencer ()
{
  //do nothing
}

void Sequencer::resetPartData ()
{
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  cacheState.part[port][channel].reset(port, channel);
	}
    }
}

void Sequencer::changeRPN_NRPN (bool isRPN,
				int port, uchar channel, uchar msb, uchar lsb,
				uchar cc1, uchar cc1Value,
				uchar cc2, uchar cc2Value)
{
  assert (0<= port);
  assert (port < MIDI::MAX_PORT);
  assert (channel < MIDI::MAX_CHANNEL);
  assert ((cc1 == 0x06 && cc2 == 0x26) ||
	  (cc1 == 0x60 && cc1Value == 0) ||
	  (cc1 == 0x61 && cc1Value == 0));

  const int savedParameterKind = cacheState.part[port][channel].getParameterKind ();
  const uchar savedRpnMSB = cacheState.part[port][channel].getRpnMSB ();
  const uchar savedRpnLSB = cacheState.part[port][channel].getRpnLSB ();
  const uchar savedNrpnMSB = cacheState.part[port][channel].getNrpnMSB ();
  const uchar savedNrpnLSB = cacheState.part[port][channel].getNrpnLSB ();

  if (isRPN)
    {
      //RPN$B$N;XDj$NHV9f$K$9$k!#(B
      if (savedRpnMSB != msb)
	controlChange (port, channel, 0x65, msb);
      if ((savedParameterKind != Part::PARAMETER_RPN && savedRpnMSB == msb) ||
	  savedRpnLSB != lsb)
	controlChange (port, channel, 0x64, lsb);
    }
  else
    {
      //NRPN$B$N;XDj$NHV9f$K$9$k!#(B
      if (savedNrpnMSB != msb)
	controlChange (port, channel, 0x63, msb);
      if ((savedParameterKind != Part::PARAMETER_NRPN && savedNrpnMSB == msb)||
	  savedNrpnLSB != lsb)
	controlChange (port, channel, 0x62, lsb);
    }

  //RPN or NRPN$B$rA`:n(B
  if (cc1 == 0x06)
    {//data entry
      controlChange (port, channel, cc1, cc1Value);
      controlChange (port, channel, cc2, cc2Value);
    }
  else
    {//incriment or decriment
      controlChange (port, channel, cc1, cc1Value);
    }
  

  if (savedParameterKind == Part::PARAMETER_NULL)
    {
      controlChange (port, channel, 0x65, 0x7f);
      controlChange (port, channel, 0x64, 0x7f);
    }
  else if (isRPN)
    {
      //$B85$KLa$9!#(B
      if (savedParameterKind == Part::PARAMETER_RPN)
	{
	  if (savedRpnMSB != msb)
	    controlChange (port, channel, 0x65, savedRpnMSB);
	  if (savedRpnLSB != lsb)
	    controlChange (port, channel, 0x64, savedRpnLSB);
	}
      else if (savedParameterKind == Part::PARAMETER_NRPN)
	{//NRPN$B$K$7$F$*$1$P$$$$$N$G!"(BMSB$B$@$1Aw$k!#(B
	  controlChange (port, channel,
			 0x63, cacheState.part[port][channel].getNrpnMSB ());
	}
    }
  else
    {
      //$B85$KLa$9!#(B
      if (savedParameterKind == Part::PARAMETER_RPN)
	{//NRPN$B$K$7$F$*$1$P$$$$$N$G!"(BMSB$B$@$1Aw$k!#(B
	  controlChange (port, channel,
			 0x65, cacheState.part[port][channel].getRpnMSB ());
	}
      else if (savedParameterKind == Part::PARAMETER_NRPN)
	{
	  if (savedNrpnMSB != msb)
	    controlChange (port, channel, 0x65, savedNrpnMSB);
	  if (savedNrpnLSB != lsb)
	    controlChange (port, channel, 0x64, savedNrpnLSB);
	}
    }
}

void Sequencer::traceAllNoteOffSoundOff ()
{
  //trace all note off
  for (int port = 0; port < MIDI::MAX_PORT; port++)
    {
      for (int channel = 0; channel < MIDI::MAX_CHANNEL; channel++)
	{
	  for (int j = 0; j < MIDI::MAX_NOTES; j++)
	    {
	      cacheState.part[port][channel].traceNoteOff (j);
	    }
	}
    }
}


void Sequencer::dumpEvent (const uchar* udata, uint32 size)
{
#ifdef DEBUG
  cerr << hex;
  for (uint i = 0; i < size; i++)
    cerr << static_cast<int>(udata[i]) << " ";
  cerr << dec << endl;
#endif
}
