/*
    SequencerStatus.h
  
    Copyright (C) 1998 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
    */
#ifndef SequencerStatus_H
#define SequencerStatus_H

#include <cassert>

#include "util/mytype.h"
#include "MIDI.h"
#include "DotData.h"

using namespace std;

class SequencerStatus
{
public:
  //////////////////////////////////////////////////////midi mode
  typedef int DeviceMode;
  //followings are not sended.
  static const DeviceMode DEVICE_UNKNOWN = 0;
  //GS Reset
  // data for GS 0xF0 0x41 dev 0x42 0x12 0x40 0x00 0x7F 0x00 0x41 0xF7
  static const DeviceMode DEVICE_GS_ON = (1 << 0);
  //GM System ON
  //data for GM 0xF0 0x7E 0x7F 0x09 0x01 0xF7
  static const DeviceMode DEVICE_GM_ON = (1 << 1);
  // data for SC-88
  //system mode set 0xF0 0x41 dev  0x42 0x12 0x00 0x00 0x7f dd   sum  0xF7
  static const DeviceMode DEVICE_SC88_ON = (1 << 2);
  static const DeviceMode DEVICE_SC88PRO_ON = (1 << 3);// data for SC-88Pro
  //XG System ON
  //0xF0 0x43 0x1n 0x4C 0x00 0x00 0x7E 0x00 0xF7
  static const DeviceMode DEVICE_XG_ON = (1 << 4);// data for XG

  ////////////////////////////////////////////////////display data
  static const int MAX_LETTERS = 32;
  static const int MAX_DISPLAY_PAGE = 10;

  /////////////////////////////////////////////////////////volume
  static const uint32 MAX_MASTER_VOLUME = 16383;

  ////////////////////////////////////////////////////////////pan
  static const char MAX_MASTER_PAN = 64;
  static const char CENTER_MASTER_PAN = 64;

private:
  //////////////////////////////////////////////////////midi mode
  DeviceMode deviceMode;//current Mode
  MIDI::PORT port;//$B8=:_$N=PNO@h$N%]!<%H(B


  ////////////////////////////////////////////////////////////mute
  //$B$3$N(Bchannel$B$,(Bmute$B$5$l$F$$$k$+(B?
  //part$BHV9f$G$J$/(Bpart$B$N<u?.%A%c%s%M%k$G$"$k$3$H$KCm0U!#(B
  bool __isMuted[MIDI::MAX_PORT][MIDI::MAX_CHANNEL];
  
  ////////////////////////////////////////////////////display data
  //letter
  char displayedLetter[MAX_LETTERS+1];
  uint32 letterTicks;
  double letterMachineTicks;
  //dot
  DotData dotData[MAX_DISPLAY_PAGE];
  uint32 dotTicks;
  double dotMachineTicks;
  //page
  int changedPage;
  uchar displayPage;//0x00-0x0A 00:bar display 01:page1 ... 0a:page10
  uchar displayTime;//0x00-0x0F 0-7.2 (sec)
  uint32 pageTicks;
  double pageMachineTicks;
  
  /////////////////////////////////////////////////////////volume
  uint32 masterVolume;//0x40 0x00 0x04

  ////////////////////////////////////////////////////////////pan
  char masterPan;//0x40 0x00 0x06

public:
  SequencerStatus ();
  void reset ();

  void setPort (MIDI::PORT port__) {port = port__;};
  MIDI::PORT getPort () const {return port;};
  
  //////////////////////////////////////////////////////midi mode
  void setDeviceMode (DeviceMode deviceMode__) {deviceMode = deviceMode__;};
  DeviceMode getDeviceMode () const {return deviceMode;};

  ////////////////////////////////////////////////////display data
  //letter
  void setDisplayedLetter (int i, uchar udata) {displayedLetter[i] = udata;};
  const char* getDisplayedLetter () const {return displayedLetter;};
  void setLetterTicks (uint32 letterTicks__, double letterMachineTicks__)
    {
      letterTicks = letterTicks__;
      letterMachineTicks = letterMachineTicks__;
    };
  uint32 getLetterTicks () const {return letterTicks;};
  double getLetterMachineTicks () const {return letterMachineTicks;};
  //dot
  void setRolandDotData (int page__,
			 const uchar rolandDotData[DotData::ROLAND_DOT_DATA_SIZE])
    {
      dotData[page__].setRolandDotData (rolandDotData);
    };
  void setDotTicks (uint32 dotTicks__, double dotMachineTicks__)
    {
      dotTicks = dotTicks__;
      dotMachineTicks = dotMachineTicks__;
    };
  const DotData& getDotData (int page__) const
    {
      assert (page__ >= 0);
      assert (page__ < MAX_DISPLAY_PAGE);
      return dotData[page__];
    };
  uint32 getDotTicks () const {return dotTicks;};
  double getDotMachineTicks () const {return dotMachineTicks;};
  //page
  void setChangedPage (int changedPage__) {changedPage = changedPage__;};
  void setDisplayPage (int displayPage__) {displayPage = displayPage__;};
  int getChangedPage () const {return changedPage;};
  uchar getDisplayPage () const {return displayPage;};
  void setDisplayTime (uchar displayTime__) {displayTime = displayTime__;};
  uchar getDisplayTime () const {return displayTime;};

  void setPageTicks (uint32 pageTicks__, double pageMachineTicks__)
    {
      pageTicks = pageTicks__;
      pageMachineTicks = pageMachineTicks__;
    };
  uint32 getPageTicks () const {return pageTicks;};
  double getPageMachineTicks () const {return pageMachineTicks;};  
  
  ////////////////////////////////////////////////////////////mute
  void setMute (MIDI::PORT port__, int channel, bool isMute__)
    {
      assert (port__ >= 0);
      assert (port__ < MIDI::MAX_PORT);
      assert (channel >= 0);
      assert (channel < MIDI::MAX_CHANNEL);
      __isMuted[port__][channel] = isMute__;
    };
  bool isMuted (MIDI::PORT port__, int channel) const
    {
      assert (port__ >= 0);
      assert (port__ < MIDI::MAX_PORT);
      assert (channel >= 0);
      assert (channel < MIDI::MAX_CHANNEL);
      return __isMuted[port__][channel];
    };

  
  /////////////////////////////////////////////////////////volume
  void setMasterVolume (uint32 masterVolume__)
    {
      assert (masterVolume__ <= MAX_MASTER_VOLUME);
      masterVolume = masterVolume__;
    };
  uint32 getMasterVolume () const {return masterVolume;}


  ////////////////////////////////////////////////////////////pan
  void setMasterPan (char masterPan__) {masterPan = masterPan__;};
  char getMasterPan () const {return masterPan;};
};
#endif// SequencerStatus_H
