/*
    SerialDevice.cc
  
    Copyright (C) 1998, 1999 Isaku Yamahata

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "SerialDevice.h"
#include "util/schedYield.h"

using namespace std;

#if defined (__hpux)
const char* const SerialDevice::DEFAULT_SERIAL_DEVICE = "/dev/tty00";
#elif defined(__linux__)
const char* const SerialDevice::DEFAULT_SERIAL_DEVICE = "/dev/ttyS0";
//const char* const SerialDevice::DEFAULT_SERIAL_DEVICE = "/dev/ttyS1";
//const char* const SerialDevice::DEFAULT_SERIAL_DEVICE = "/dev/cua0";
#else
const char* const SerialDevice::DEFAULT_SERIAL_DEVICE = "/dev/ttya";
#endif

const int SerialDevice::DEFAULT_BPS = 38400;
const speed_t SerialDevice::B31250;

SerialDevice::SerialDevice ()
{
  open (DEFAULT_SERIAL_DEVICE, DEFAULT_BPS);
}

SerialDevice::SerialDevice (const string& device)
{
  if (device == "")
    open (DEFAULT_SERIAL_DEVICE, DEFAULT_BPS);
  else
    open (device, DEFAULT_BPS);
}

SerialDevice::SerialDevice (const string& device, int bps)
{
  open (device, bps);
}

SerialDevice::~SerialDevice ()
{
  if (isatty (serialfd))
    {
      if (tcsetattr (serialfd, TCSAFLUSH, &oldTermios) < 0)
	{//tty$B$N>uBV$r$7$h$&0JA0$KLa$;$J$+$C$?!#(B
	  cerr << "SerialDevice::~SerialDevice:Can't set addribute tty."
	       << ".:" << strerror (errno) << endl;
	}
    }
  close (serialfd);
}

void SerialDevice::open (const string& device, int bps)
{
  //these codes is stolen from midiplay
  speed_t speed;
  
  switch(bps)
    {
    case 38400:
      speed = B38400;
      break;
    case 31250:
      speed = B31250;
      break;
    default:
      cerr << "SerialDevice::SerialDevice: Bogus speed " << bps
	   << " is given." << endl;
      throw BogusBPS ();
      break;
    }
  
  serialfd = std::open(device.c_str (), O_RDWR | O_NDELAY);
  if (serialfd == -1)
    {
      cerr << "SerialDevice::SerialDevice: Can't open device "
	   << device << ".:" << strerror (errno) << endl;
      throw CanNotOpenDevice ();
    }

  if (isatty (serialfd))
    {
      termios tio;
      if (tcgetattr (serialfd, &tio) < 0)
	{
	  cerr << "SerialDevice::SerialDevice:Can't get attribute of"
	       <<  device << ".:" << strerror (errno) << endl;
	  close (serialfd);
	  throw CanNotGetDeviceAttribute ();
	}
      oldTermios = tio;//close$B$N:]$K85$KLa$90Y$K@_Dj$r3P$($F$*$/!#(B
      tio.c_iflag = 0;
      tio.c_oflag = 0;
      tio.c_cflag = CS8 | CREAD | CLOCAL;
      tio.c_lflag = 0;
      cfsetispeed (&tio, speed);
      cfsetospeed (&tio, speed);
      if (tcsetattr (serialfd, TCSANOW, &tio) < 0)
	{
	  cerr << "SerialDevice::SerialDevice: Can't set attribute of"
	       << device << ".:" << strerror (errno) << endl;
	  close (serialfd);
	  throw CanNotSetDeviceAttribute ();
	}
    }
}



void SerialDevice::write (const uchar* data, ssize_t length)
{
#ifdef DEBUG
  //cerr << "SerialDevice::write called:length=" << length << endl;
#endif

  ssize_t wroteSize = 0;
  while (wroteSize < length)
    {
      const int ret = std::write (serialfd, data+wroteSize, length-wroteSize);
      if (ret == -1)
	{
	  if (errno == EAGAIN)
	    {//XXX$B$7$P$i$/BT$F$P$$$$$N$@$,!D!#(B
#if 0
	      if (fsync (serialfd) == -1)
		{
		  perror ("SerialDevice::write:fsync Can not fsync.fatal error.");
		  exit (1);
		}
#else
	      schedYield ();
#endif
	      continue;
	    }

	  if (errno == EINTR)
	    continue;
	  
	  //UNEXPECTED ERROR!
	  //XXX $B$b$C$H$^$7$J%(%i!<=hM}$NJ}K!$,$"$kH&!#(B
	  perror ("SerialDevice::write: Can not write.fatal error.");
	  exit (1);
	}
      
      wroteSize += ret;
    }
}
